@@ -1,9 +1,8 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gedit-taglist-plugin-parser.c
- * This file is part of the gedit taglist plugin
+ * This file is part of gedit
  *
- * Copyright (C) 2002 Paolo Maggi 
+ * Copyright (C) 2002-2005 - Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +21,15 @@
  */
  
 /*
- * Modified by the gedit Team, 2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 2002-2005. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
+ *
+ * $Id: gedit-taglist-plugin-parser.c 5351 2006-12-09 13:26:06Z paolo $
  */
 
+/* FIXME: we should rewrite the parser to avoid using DOM */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -45,13 +48,16 @@
 
 #include "gedit-taglist-plugin-parser.h"
 
+// FIXME
 #define USER_GEDIT_TAGLIST_PLUGIN_LOCATION ".gedit-2/plugins/taglist/"
 
 TagList *taglist = NULL;
+static gint taglist_ref_count = 0;
 
 static gboolean	 parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
 static gboolean	 parse_tag_group (TagGroup *tg, const gchar *fn, 
-				  xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
+				  xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
+				  gboolean sort);
 static TagGroup* get_tag_group (const gchar* filename, xmlDocPtr doc, 
 				xmlNsPtr ns, xmlNodePtr cur);
 static TagList* lookup_best_lang (TagList *taglist, const gchar *filename, 
@@ -66,7 +72,7 @@ static gboolean
 parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
 {
 	/*
-	gedit_debug (DEBUG_PLUGINS, "  Tag name: %s", tag->name);
+	gedit_debug_message (DEBUG_PLUGINS, "  Tag name: %s", tag->name);
 	*/
 	/* We don't care what the top level element name is */
 	cur = cur->xmlChildrenNode;
@@ -78,7 +84,7 @@ parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 		{			
 			tag->begin = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
 			/*
-			gedit_debug (DEBUG_PLUGINS, "    - Begin: %s", tag->begin);
+			gedit_debug_message (DEBUG_PLUGINS, "    - Begin: %s", tag->begin);
 			*/
 		}
 
@@ -87,7 +93,7 @@ parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 		{
 			tag->end = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
 			/*
-			gedit_debug (DEBUG_PLUGINS, "    - End: %s", tag->end);
+			gedit_debug_message (DEBUG_PLUGINS, "    - End: %s", tag->end);
 			*/
 		}
 
@@ -100,11 +106,20 @@ parse_tag (Tag *tag, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	return TRUE;
 }
 
+static gint
+tags_cmp (gconstpointer a, gconstpointer b)
+{
+	gchar *tag_a = (gchar*)((Tag *)a)->name;
+	gchar *tag_b = (gchar*)((Tag *)b)->name;
+	
+	return g_utf8_collate (tag_a, tag_b);
+}
+
 static gboolean
 parse_tag_group (TagGroup *tg, const gchar* fn, xmlDocPtr doc, 
-		 xmlNsPtr ns, xmlNodePtr cur) 
+		 xmlNsPtr ns, xmlNodePtr cur, gboolean sort) 
 {
-	gedit_debug (DEBUG_PLUGINS, "Parse TagGroup: %s", tg->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Parse TagGroup: %s", tg->name);
 
 	/* We don't care what the top level element name is */
     	cur = cur->xmlChildrenNode;
@@ -142,8 +157,8 @@ parse_tag_group (TagGroup *tg, const gchar* fn, xmlDocPtr doc,
 				/* Parse Tag */
 				if (parse_tag (tag, doc, ns, cur))
 				{
-					/* Append Tag to TagGroup */
-					tg->tags = g_list_append (tg->tags, tag);
+					/* Prepend Tag to TagGroup */
+					tg->tags = g_list_prepend (tg->tags, tag);
 				}
 				else
 				{
@@ -162,6 +177,11 @@ parse_tag_group (TagGroup *tg, const gchar* fn, xmlDocPtr doc,
 		cur = cur->next;		
 	}
 
+	if (sort)
+		tg->tags = g_list_sort (tg->tags, tags_cmp);
+	else
+		tg->tags = g_list_reverse (tg->tags);	
+	
 	return TRUE;
 }
 
@@ -170,67 +190,86 @@ static TagGroup*
 get_tag_group (const gchar* filename, xmlDocPtr doc, 
 		 xmlNsPtr ns, xmlNodePtr cur) 
 {
-			TagGroup *tag_group;
+	TagGroup *tag_group;
+	gchar *sort_str;
+	gboolean sort = FALSE;
 
+	tag_group = g_new0 (TagGroup, 1);
 
-			tag_group = g_new0 (TagGroup, 1);
+	/* Get TagGroup name */
+	tag_group->name = xmlGetProp (cur, (const xmlChar *) "name");
 
-			/* Get TagGroup name */
-			tag_group->name = xmlGetProp (cur, (const xmlChar *) "name");
+	sort_str = (gchar *)xmlGetProp (cur, (const xmlChar *) "sort");
+	
+	if ((sort_str != NULL) &&
+	    ((g_ascii_strcasecmp (sort_str, "yes") == 0) ||
+	     (g_ascii_strcasecmp (sort_str, "true") == 0) ||
+	     (g_ascii_strcasecmp (sort_str, "1") == 0)))
+	{
+		sort = TRUE;
+	}
+	
+	if (tag_group->name == NULL)
+	{
+		/* Error: No name */
+		g_warning ("The tag list file '%s' is of the wrong type, "
+		   "TagGroup without name.", filename);
 
-			if (tag_group->name == NULL)
+		g_free (tag_group);
+	}
+	else
+	{
+		/* Name found */
+		gboolean exists = FALSE;
+		GList *t = taglist->tag_groups;
+		
+		/* Check if the tag group already exists */
+		while (t && !exists)
+		{
+			gchar *tgn = (gchar*)((TagGroup*)(t->data))->name;
+			
+			if (strcmp (tgn, (gchar*)tag_group->name) == 0)
 			{
-				/* Error: No name */
-				g_warning ("The tag list file '%s' is of the wrong type, "
-				   "TagGroup without name.", filename);
+				gedit_debug_message (DEBUG_PLUGINS, 
+					     "Tag group '%s' already exists.", tgn);
+				
+				exists = TRUE;
 
-				g_free (tag_group);
+				free_tag_group (tag_group);
+			}
+			
+			t = g_list_next (t);		
+		}
+
+		if (!exists)
+		{				
+			/* Parse tag group */
+			if (parse_tag_group (tag_group, filename, doc, ns, cur, sort))
+			{
+				return tag_group;
 			}
 			else
 			{
-				/* Name found */
-				gboolean exists = FALSE;
-				GList *t = taglist->tag_groups;
-				
-				/* Check if the tag group already exists */
-				while (t && !exists)
-				{
-					gchar *tgn = (gchar*)((TagGroup*)(t->data))->name;
-					
-					if (strcmp (tgn, (gchar*)tag_group->name) == 0)
-					{
-						gedit_debug (DEBUG_PLUGINS, 
-							     "Tag group '%s' already exists.", tgn);
-						
-						exists = TRUE;
+				/* Error parsing TagGroup */
+				g_warning ("The tag list file '%s' is of the wrong type, "
+		   			   "error parsing TagGroup '%s'.", 
+					   filename, tag_group->name);
 
-						free_tag_group (tag_group);
-					}
-					
-					t = g_list_next (t);		
-				}
-
-				if (!exists)
-				{				
-					/* Parse tag group */
-					if (parse_tag_group (tag_group, filename, doc, ns, cur))
-					{
-						return tag_group;
-					}
-					else
-					{
-						/* Error parsing TagGroup */
-						g_warning ("The tag list file '%s' is of the wrong type, "
-				   			   "error parsing TagGroup '%s'.", 
-							   filename, tag_group->name);
-
-						free_tag_group (tag_group);
-					}
-				}
+				free_tag_group (tag_group);
 			}
+		}
+	}
 	return NULL;
 }
 
+static gint
+groups_cmp (gconstpointer a, gconstpointer b)
+{
+	gchar *g_a = (gchar *)((TagGroup *)a)->name;
+	gchar *g_b = (gchar *)((TagGroup *)b)->name;
+	
+	return g_utf8_collate (g_a, g_b);
+}
 
 /* 
  *  tags file is localized by intltool-merge below.
@@ -286,14 +325,14 @@ lookup_best_lang (TagList *taglist, const gchar *filename,
 			cur_lanking = 1;
 
 			/* 
-			 * When found a new TagGroup, append the best 
+			 * When found a new TagGroup, prepend the best 
 			 * tag_group to taglist. In the current intltool-merge, 
 			 * the first section is the default lang NULL.
 			 */
 			if (lang == NULL) {
 				if (best_tag_group != NULL) {
 					taglist->tag_groups = 
-					g_list_append (taglist->tag_groups, best_tag_group);
+					g_list_prepend (taglist->tag_groups, best_tag_group);
 				}
 
 				best_tag_group = NULL;
@@ -364,16 +403,17 @@ lookup_best_lang (TagList *taglist, const gchar *filename,
 		cur = cur->next;
     	} /* End of while (cur != NULL) */
 
-	/* Append TagGroup to TagList */
+	/* Prepend TagGroup to TagList */
 	if (best_tag_group != NULL) {
 		taglist->tag_groups = 
-			g_list_append (taglist->tag_groups, best_tag_group);
+			g_list_prepend (taglist->tag_groups, best_tag_group);
 	}
 
+	taglist->tag_groups = g_list_sort (taglist->tag_groups, groups_cmp);
+	
 	return taglist;
 }
 
-
 static TagList *
 parse_taglist_file (const gchar* filename)
 {
@@ -382,7 +422,7 @@ parse_taglist_file (const gchar* filename)
 	xmlNsPtr ns;
 	xmlNodePtr cur;
 
-	gedit_debug (DEBUG_PLUGINS, "Parse file: %s", filename);
+	gedit_debug_message (DEBUG_PLUGINS, "Parse file: %s", filename);
 
 	xmlKeepBlanksDefault (0);
 
@@ -442,7 +482,7 @@ parse_taglist_file (const gchar* filename)
 
 	xmlFreeDoc (doc);
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "END");
 
 	return taglist;
 }
@@ -452,7 +492,7 @@ static void
 free_tag (Tag *tag)
 {
 	/*
-	gedit_debug (DEBUG_PLUGINS, "Tag: %s", tag->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Tag: %s", tag->name);
 	*/
 	g_return_if_fail (tag != NULL);
 
@@ -470,7 +510,7 @@ free_tag (Tag *tag)
 static void
 free_tag_group (TagGroup *tag_group)
 {
-	gedit_debug (DEBUG_PLUGINS, "Tag group: %s", tag_group->name);
+	gedit_debug_message (DEBUG_PLUGINS, "Tag group: %s", tag_group->name);
 
 	g_return_if_fail (tag_group != NULL);
 
@@ -487,17 +527,23 @@ free_tag_group (TagGroup *tag_group)
 	
 	g_free (tag_group);
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "END");
 }
 
 void
 free_taglist (void)
 {
-	gedit_debug (DEBUG_PLUGINS, "");
-
+	gedit_debug_message (DEBUG_PLUGINS, "ref_count: %d", taglist_ref_count);
+	
 	if (taglist == NULL)
 		return;
-	
+		
+	g_return_if_fail (taglist_ref_count > 0);
+
+	--taglist_ref_count;
+	if (taglist_ref_count > 0)
+		return;
+		
 	while (taglist->tag_groups)
 	{
 		free_tag_group ((TagGroup *)taglist->tag_groups->data);
@@ -511,7 +557,7 @@ free_taglist (void)
 
 	taglist = NULL;
 
-	gedit_debug (DEBUG_PLUGINS, "END");
+	gedit_debug_message (DEBUG_PLUGINS, "Really freed");
 }
 
 static TagList * 
@@ -520,13 +566,13 @@ parse_taglist_dir (const gchar *dir)
 	DIR *d;
 	struct dirent *e;
 
-	gedit_debug (DEBUG_PLUGINS, "DIR: %s", dir);
+	gedit_debug_message (DEBUG_PLUGINS, "DIR: %s", dir);
 
 	d = opendir (dir);
 	
 	if (d == NULL)
 	{		
-		gedit_debug (DEBUG_PLUGINS, "%s", strerror (errno));
+		gedit_debug_message (DEBUG_PLUGINS, "%s", strerror (errno));
 		return taglist;
 	}
 	
@@ -548,10 +594,15 @@ TagList* create_taglist (void)
 {
 	gchar const * const home = g_get_home_dir ();
 
-	gedit_debug (DEBUG_PLUGINS, "");
-
-	g_return_val_if_fail (taglist == NULL, taglist);
+	gedit_debug_message (DEBUG_PLUGINS, "ref_count: %d", taglist_ref_count);
 
+	if (taglist_ref_count > 0)
+	{		
+		++taglist_ref_count;
+		
+		return taglist;
+	}
+		
 	/* load user's taglists */
 	if (home != NULL)
 	{
@@ -566,5 +617,8 @@ TagList* create_taglist (void)
 	/* load system's taglists */	
 	parse_taglist_dir (GEDIT_TAGLIST_DIR);
 
+	++taglist_ref_count;
+	g_return_val_if_fail (taglist_ref_count == 1, taglist);
+	
 	return taglist;
 }
